%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%% Demonstration Tool for RTHS %%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%IISL, Purdue University%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Original version:
%       Gaby Ou  (May 2013)

% Modified by :
%       Rachel Chicchi
%       Daniel Gomez
%       Scott Wonderly
% (December 6, 2013)

% Acknowledgement to:
%       Amin Maghareh (Dec 2013) 


%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%              Building Information         %
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
clc, clear, close all, warning off

M  	 = 155.6;					% mass per floor [N/(m/sec2)]
K    = 1.004922e+04;		    % N/m
Zeta = 0.02;
C    = 2*M*Zeta*sqrt(K/M);
wn_T = sqrt(K/M)/(2*pi());
wd_T = wn_T*sqrt(1-Zeta^2);

disp(['Entire Structure Damped Natural Frequency is: ' num2str(round(100*wd_T)/100) ' Hz'])
ndof = length(M);

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%        Base Isolation Information         %
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
Option = 1;

switch Option
case 1
     m_b =  10;   
     k_b =  K/200; 
     zeta_b = .5;         
     c_b    = 2*zeta_b*sqrt(k_b*m_b);
case 2
     m_b =  10;   
     k_b =  K/20;   
     zeta_b = .5;          
     c_b = 2*zeta_b*sqrt(k_b*m_b);
case 3      
     m_b =  10000;  
     k_b =  K/2;  
     zeta_b = .5; 
     c_b= 2*zeta_b*sqrt(k_b*m_b);  
end
ndofb = 2; 

%------------------------------------
% RTHS Reference system
% Input:   1) Ground Displacement, [m]
%          2) Ground Velocity,     [m/sec]
% Outputs: 1) Total displacement of ROOF
%          2) Total displacement of BI
%          3) Total velocity of ROOF
%          4) Total velocity of BI
%          5) Total acceleration of ROOF
%          6) Total acceleration of BI


% Steel Building (Numerical Substructure)
ndof = 2;
MN = M;
KN = K; 
CN = C;

% Base Isolator (Experimental Substructure)
ME = m_b; 
KE = k_b;
CE = c_b;
zeta_e = zeta_b;

% REFERENCE SYSTEM (Experimental and Numerical Substructure)
AA_REF=[zeros(ndof) eye(ndof); -KN/MN KN/MN -CN/MN CN/MN; KN/ME -(KN+KE)/ME CN/ME -(CN+CE)/ME]; 
BB_REF=[zeros(ndof+1,2); KE/ME CE/ME];
CC_REF=[eye(ndof*2); -KN/MN KN/MN -CN/MN CN/MN; KN/ME -(KN+KE)/ME CN/ME -(CN+CE)/ME]; 
DD_REF=[zeros(ndof*2+1,2); KE/ME CE/ME];

sys_REF = ss(AA_REF,BB_REF,CC_REF,DD_REF);

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%            Partitioning of RTHS            %
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
R_ME = ME/(MN+ME);

% CHECK STABILITY OF Partition
% if (R_ME > 0.57)
%     disp(' ')
%     f = warndlg('Unacceptable Mass Partition !', 'CHECK STABILITY');
% end

wn_E=sqrt(KE/ME)/(2*pi);
wd_E=wn_E*sqrt(1-zeta_e^2);

disp(['Experimental Substructure Damped Natural Frequency is: ' num2str(round(100*wd_E)/100) ' Hz'])

% STATE SPACE MODEL(Numerical Substructure)
AA_N=[0 1; -KN/MN -CN/MN];
BB_N=[0 0; KN/MN CN/MN];
CC_N=[1 0; 0 1; -KN/MN -CN/MN];
DD_N=[0 0; 0 0; KN/MN CN/MN];
sys_N = ss(AA_N,BB_N,CC_N,DD_N);

% STATE SPACE MODEL(Experimental Substructure)
AA_E = [0 1; -KE/ME -CE/ME];
BB_E = [0 0 0; KE/ME CE/ME 1/ME];
CC_E = [1 0; 0 1;-KE/ME -CE/ME];
DD_E = [zeros(2,3); KE/ME CE/ME 1/ME];
sys_E = ss(AA_E,BB_E,CC_E,DD_E);

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%      Actuator Dynamics and Control        %
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
C_sw = 4;             % actuator compensation switch choose from 1-8
load (['Level_' num2str(C_sw) '_comp.mat']);
load ACT_DYN;          % actuator dyanmcis

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%      General Simulation Parameter          %
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
dt_rths = 1/1024;

E_sw = 1;             % Earthquake switch 1: el centro 2: kobe 3: morgan
eq_intensity = 1;   % EQ Intensity

switch E_sw   % Earthquake Ending Time
case 1
    Tmax = 29.36;              
case 2
    Tmax = 50.94;
case 3
    Tmax = 39.974;
end
       
OPTIONS = simset('solver','ode5','FixedStep',dt_rths);
sim('BI_RTHS_simul',[0 Tmax],OPTIONS,[]); 
% 
% 
% %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% %     Data Analysis: RTHS vs Entire Structure     %
% %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

% figure 1: plot relative displacement response from FULL RTHS, IDEAL RTHS and REFERENCE STRUCTURE
scrsz = get(0,'ScreenSize');
figure(1),
set(1,'Position',[5 scrsz(4)*(0.53) scrsz(3)*(1/3-1/100) scrsz(4)*(1/2-1/8)])

subplot(2,2,1)
plot(t,REL_response_EXPER_act(:,2),'c','linewidth',1.5), hold on
plot(t,REL_response_EXPER(:,2),'b','linewidth',1.5), 
plot(t,REF_rel(:,2),'r--','linewidth',1), grid on
title('EXPERIMENTAL SUBSTRUCTURE','fontweight','bold')
ylabel('Rel. Displacement, [m]','FontSize',9,'fontweight','bold')
aa =legend('RTHS Full Case','RTHS Ideal Case','Reference Case',4);
set(aa,'fontsize',8)

REL_response_NUM = TOT_response_NUM(:,1) - Ground_quake(:,1);

subplot(2,2,2)
plot(t,REL_response_NUM_act(:,1),'c','linewidth',1.5), hold on
plot(t,REL_response_NUM(:,1),'b','linewidth',1.5), 
plot(t,REF_rel(:,1),'r--','linewidth',1), grid on
title('NUMERICAL SUBSTRUCTURE','fontweight','bold')

subplot(2,2,3)
plot(t,REL_response_EXPER_act(:,2),'c','linewidth',1.5), hold on
plot(t,REL_response_EXPER(:,2),'b','linewidth',1.5), 
plot(t,REF_rel(:,2),'r--','linewidth',1), grid on
xlim([5 15])
xlabel('Time, [sec]','FontSize',9,'fontweight','bold')
ylabel('Rel. Displacement, [m]','FontSize',9,'fontweight','bold')

subplot(2,2,4)
plot(t,REL_response_NUM_act(:,1),'c','linewidth',1.5), hold on
plot(t,REL_response_NUM(:,1),'b','linewidth',1.5), 
plot(t,REF_rel(:,1),'r--','linewidth',1), grid on
xlim([5 15])
xlabel('Time, [sec]','FontSize',9,'fontweight','bold')
saveas(1,'disp','jpeg')

%figure 2: plot relative velocity response from FULL RTHS, IDEAL RTHS and REFERENCE STRUCTURE
figure(2)
set(2,'Position',[5+scrsz(3)/3 scrsz(4)*(0.53) scrsz(3)*(1/3-1/100) scrsz(4)*(1/2-1/8)])
subplot(2,2,1)
REL_response_EXPER = TOT_response_EXPER(:,2) - Ground_quake(:,2);

plot(t,REL_response_EXPER_act(:,1),'c','linewidth',1.5), hold on
plot(t,REL_response_EXPER(:,1),'b','linewidth',1.5), 
plot(t,REF_rel(:,4),'r--','linewidth',1), grid on
title('EXPERIMENTAL SUBSTRUCTURE','fontweight','bold')
ylabel('Rel. Velocity, [m/s]','FontSize',9,'fontweight','bold')
aa =legend('RTHS Full Case','RTHS Ideal Case','Reference Case',4);
set(aa,'fontsize',8)

REL_response_NUM = TOT_response_NUM(:,2) - Ground_quake(:,2);
% REL_response_NUM_act = TOT_response_NUM_act(:,2) - Ground_quake(:,2);

subplot(2,2,2)
plot(t,REL_response_NUM_act(:,2),'c','linewidth',1.5), hold on
plot(t,REL_response_NUM(:,1),'b','linewidth',1.5), hold on
plot(t,REF_rel(:,3),'r--','linewidth',1), grid on
title('NUMERICAL SUBSTRUCTURE','fontweight','bold')

subplot(2,2,3)
plot(t,REL_response_EXPER_act(:,1),'c','linewidth',1.5), hold on
plot(t,REL_response_EXPER(:,1),'b','linewidth',1.5), hold on
plot(t,REF_rel(:,4),'r--','linewidth',1), grid on
xlim([5 15])
xlabel('Time, [sec]','FontSize',9,'fontweight','bold')
ylabel('Rel. Velocity, [m/s]','FontSize',9,'fontweight','bold')

subplot(2,2,4)
plot(t,REL_response_NUM_act(:,2),'c','linewidth',1.5), hold on
plot(t,REL_response_NUM(:,1),'b','linewidth',1.5), hold on
plot(t,REF_rel(:,3),'r--','linewidth',1), grid on
xlim([5 15])
xlabel('Time, [sec]','FontSize',9,'fontweight','bold')
saveas(2,'vel','jpeg')

%figure 3: plot total Acceleration response from FULL RTHS, IDEAL RTHS and REFERENCE STRUCTURE
figure(3)
set(3,'Position',[5+scrsz(3)*(2/3) scrsz(4)*(0.53) scrsz(3)*(1/3-1/100) scrsz(4)*(1/2-1/8)])
subplot(2,2,1)
plot(t,TOT_response_EXPER_act(:,3),'c','linewidth',1.5), hold on
plot(t,TOT_response_EXPER(:,3),'b','linewidth',1.5), 
plot(t,REF_tot(:,6),'r--','linewidth',1), grid on
title('EXPERIMENTAL SUBSTRUCTURE','fontweight','bold')
ylabel('Total Acceleration, [m/s^2]','FontSize',9,'fontweight','bold')
aa =legend('RTHS Full Case','RTHS Ideal Case','Reference Case',4);
set(aa,'fontsize',8)

subplot(2,2,2)
plot(t,TOT_response_NUM_act(:,3),'c','linewidth',1.5), hold on
plot(t,TOT_response_NUM(:,3),'b','linewidth',1.5), hold on
plot(t,REF_tot(:,5),'r--','linewidth',1), grid on
title('NUMERICAL SUBSTRUCTURE','fontweight','bold')

subplot(2,2,3)
plot(t,TOT_response_EXPER_act(:,3),'c','linewidth',1.5), hold on
plot(t,TOT_response_EXPER(:,3),'b','linewidth',1.5), hold on
plot(t,REF_tot(:,6),'r--','linewidth',1), grid on
xlim([5 15])
xlabel('Time, [sec]','FontSize',9,'fontweight','bold')
ylabel('Total Acceleration, [m/s^2]','FontSize',9,'fontweight','bold')

subplot(2,2,4)
plot(t,TOT_response_NUM_act(:,3),'c','linewidth',1.5), hold on
plot(t,TOT_response_NUM(:,3),'b','linewidth',1.5), hold on
plot(t,REF_tot(:,5),'r--','linewidth',1), grid on
xlim([5 15])
xlabel('Time, [sec]','FontSize',9,'fontweight','bold')
saveas(3,'accel','jpeg')

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%     Data Analysis: FULL RTHS vs IDEAL RTHS      %
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

%figure 4: plot actuator control performance
figure(4)
set(4,'Position',[5 scrsz(4)*(1/20) scrsz(3)*(1/3-1/100) scrsz(4)*(1/2-1/8)])
subplot(211)
plot(t,DM,'r','linewidth',1.5), hold on
plot(t,DD,'b','linewidth',1.5), grid on
ylabel('Displacement, [m]','FontSize',9,'fontweight','bold')
aa =legend('Actuator Displacement','Command Displacement',4);
set(aa,'fontsize',8)
title('ACTUATOR DISPLACEMENT CONTROL PERFORMANCE','fontweight','bold')

subplot(212)
plot(t,DM,'r','linewidth',1.5), hold on
plot(t,DD,'b','linewidth',1.5), grid on
ylabel('Displacement, [m]','FontSize',9,'fontweight','bold')
xlabel('Time, [sec]','FontSize',9,'fontweight','bold')
xlim([5 10])
saveas(4,'Actuator','jpeg')

%figure 5: plot FULL RTHS Restoring force and IDEAL RTHS Restoring force
figure(5)
set(5,'Position',[5+scrsz(3)*(1/3) scrsz(4)*(1/20) scrsz(3)*(1/3-1/100) scrsz(4)*(1/2-1/8)])
subplot(2,1,1)
plot(t,Restoring_Force_act(:,1),'c','linewidth',1.5), hold on,grid on
plot(t,Restoring_Force(:,1),'b','linewidth',1.5), grid on
ylabel('Restoring Force, [N]','FontSize',9,'fontweight','bold')
title('RESTORING FORCE','fontweight','bold')
aa =legend('RTHS Full Case','RTHS Ideal Case',4);
set(aa,'fontsize',8)

subplot(2,1,2)
plot(t,Restoring_Force_act(:,1),'c','linewidth',1.5), hold on,grid on
plot(t,Restoring_Force(:,1),'b','linewidth',1.5), grid on
ylabel('Restoring Force, [N]','FontSize',9,'fontweight','bold')
xlim([5 15])
xlabel('Time, [sec]','FontSize',9,'fontweight','bold')
saveas(5,'Force','jpeg')

